import curses

from typing import Callable, Dict

from colored import attr, fg

from .custom_string import String
from .renderer import Renderer


class AutoComplete(Renderer):
    def __init__(self, commands: Dict[str, Callable]):
        super(AutoComplete, self).__init__()
        self._index = 0
        self._buffer = [" "] * self.width
        self.text = ""

        self.commands = commands

    def delete_char(self):
        if len(self.text) > 1:
            self.text = self.text[:-1]

    def get_actual_text(self):
        return self.text[1:]

    def execute_command_if_matched(self):
        for command, func in self.commands.items():
            if command.startswith(self.get_actual_text()):
                return func()

    def handle_key(self, key):
        guessed_command = self.guess_command()
        if key.code == curses.KEY_RIGHT:
            if guessed_command:
                self.text = ":" + guessed_command

    def guess_command(self):
        text = self.get_actual_text()
        if len(text) == 0:
            return ""

        for command in self.commands:
            if command.startswith(text.lower()):
                return command

        return ""

    def draw(self, echo_func: Callable):
        guessed_command = self.guess_command()
        text = self.get_actual_text()
        unmatched_part = guessed_command[len(text):]

        to_print = (String(self.text, fg=attr('reset')) + String(unmatched_part, fg=fg('grey_50'))).pad(self.width)[-self.width:]

        echo_func(self.x, self.y, to_print)



from typing import Callable

from colored import fg, bg

from .custom_string import Style
from .renderer import ScrollableRenderer

MAX_RECORDS = 100


class Logger(ScrollableRenderer):
    def __init__(self):
        super(Logger, self).__init__([])
        self._halt_debug = False
        self.shown = False

        self.halt_buffer = []

    def halt_debug(self):
        self._halt_debug = True

    def continue_debug(self):
        for msg in self.halt_buffer:
            self._add_message(msg)

        self.halt_buffer = []
        self._halt_debug = False

    def _add_message(self, msg: str):
        self.data = (self._data + [msg])[-MAX_RECORDS:]

    def on_data_set(self, new_value):
        self._displayed_data_start = max(len(self.displayed_data) - self.height + 1, 0)
        self.selected_row = len(self.displayed_data) - 1

    def debug(self, msg: str):
        if self._halt_debug:
            self.halt_buffer.append(msg)
            return

        self._add_message(msg)

    def draw(self, echo_func: Callable):
        if not self.shown:
            return
            
        self._curr_row = 0
        for i, log in enumerate(self.displayed_data[self.displayed_data_start:self.displayed_data_end]):
            actual_i = self.displayed_data_start + i
            row_style = Style()
            if self.selected_row == actual_i:
                row_style = Style(fg=fg("black"), bg=bg("light_gray"))

            log = log + " " * (self.width - len(log))
            echo_func(self.x, self.y + self._curr_row, self._trim_row(log).with_style(row_style))
            self._curr_row += 1
        
        super().draw(echo_func)


logger = Logger()



import os
import sys
from typing import List
import tempfile
from subprocess import call

from blessed import Terminal
from colored import attr

from .custom_string import String
from .renderer import Renderer


EDITOR = os.environ.get('EDITOR', 'vim')


def flush():
    sys.stdout.flush()


class App:
    def __init__(self):
        self.term = Terminal()
        self.render_list = []
        self.should_run = True

        self.buffer = [[' ' for x in range(self.term.width)] for y in range(self.term.height)]

    def dump_to_screen(self):
        for line_num, line in enumerate(self.buffer):
            sys.stdout.write(self.term.move_xy(0, line_num) + "".join(line) + attr("reset"))

        sys.stdout.write(attr("reset"))

    def resize(self):
        self.buffer = [[' ' for x in range(self.term.width)] for y in range(self.term.height)]

    def echo(self, x, y, to_print):
        if isinstance(to_print, str):
            to_print = String(to_print)

        if y >= len(self.buffer) or x + len(to_print) > len(self.buffer[0]):
            return

        line = self.buffer[y]
        to_print.reset_style_on_end()
        for i, char in enumerate(to_print):
            actual_x = x + i
            line[actual_x] = str(char)

    def clear(self):
        sys.stdout.write(self.term.clear)
        self.buffer = [[' ' for x in range(self.term.width)] for y in range(self.term.height)]

    def add_to_render(self, renderer: Renderer):
        self.render_list.append(renderer)

    def add_multiple(self, renders: List[Renderer]):
        self.render_list.extend(renders)

    @classmethod
    def get_editor_input(cls, initial_text: str = ""):
        with tempfile.NamedTemporaryFile(suffix=".tmp") as tf:
            tf.write(initial_text.encode())
            tf.flush()
            call([EDITOR, tf.name])

            # do the parsing with `tf` using regular File operations.
            # for instance:
            tf.seek(0)
            return tf.read().decode()

    def interactive_run(self):
        with self.term.fullscreen():
            with self.term.cbreak(), self.term.raw():
                while self.should_run:
                    # self.echo(0, 0, self.term.clear)
                    with self.term.hidden_cursor():
                        for renderer in self.render_list:
                            renderer.draw(self.echo)

                        self.dump_to_screen()
                        flush()

                        key = self.term.inkey(timeout=0.1)
                        if key in [chr(3)]:
                            break

                        yield key



from pkg_resources import get_distribution

try:
    package = get_distribution(__package__)
    __version__ = package.version
    if __version__ == '0':
        __version__ = 'development'

    else:
        __version__ = 'v' + __version__

except:
    __version__ = 'unknown'
